'' =================================================================================================
''
''   File....... hc-8+_wii_demo.spin
''   Purpose.... Interface demo for Nintendo Wii controllers
''               -- requires EFX-TEK GameOn adapter (#99001)
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2012 Jon McPhalen
''               -- see below for terms of use
''   E-mail.....  
''   Started.... 
''   Updated.... 13 JUL 2012
''
'' =================================================================================================

{

  Note: Before installing the GameOn adapter you must set the A0 and SM option switches to
        the OFF position.
}
   

con

  PCB_REV = "C"


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000
  

con

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom
  SCL     = 28

  OUT_EN  = 27                                                  ' outputs enable pin (low)     
  
  SIO     = 26                                                  ' TTL serial io
  
  RX2     = 23                                                  ' RS-485 port
  RX2_EN  = 22
  TX2_EN  = 21
  TX2     = 20

  R_LED   = 18                                                  ' bi-color LED
  G_LED   = 17

  OUT7    = 15                                                  ' outputs
  OUT6    = 14   
  OUT5    = 13 
  OUT4    = 12
  OUT3    = 11
  OUT2    = 10   
  OUT1    =  9 
  OUT0    =  8
          
  OPT_BR  =  7                                                  ' options swithes
  OPT_A1  =  6   
  OPT_A0  =  5
  OPT_SM  =  4

  DMX_A8  =  3                                                  ' high-bit off DMX address

  DO_165  =  2                                                  ' control lines for 74x165
  CLK_165 =  1 
  LD_165  =  0


  ' for EFX-TEK GameOn adapter module (#99001)

  WII_DAT =  OPT_A0
  WII_CLK =  OPT_SM
  

  ' for EFX-TEK uSD adapter module (#99002)
  ' -- must open all option switches when installed

  SD_CS   =  OPT_BR                                       
  SD_DI   =  OPT_A1  
  SD_CLK  =  OPT_A0  
  SD_DO   =  OPT_SM  


con

   #0, OFF, RED, GRN, YEL                                       ' for r/g led

   #0, NOCON, RX, TX, RXTX                                      ' for rs-485

   #0, LSBFIRST, MSBFIRST                                       ' for 74x165 shift-register                                               


con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"                                   ' terminal IO
  nc   : "efx_nunchuk"                                          ' Wii Nunchuk controller
  cc   : "efx_classic"                                          ' Wii Classic controller

  
var


dat

NC_Screen               byte    "====================", CR
                        byte    " Wii Nunchuk Reader ", CR
                        byte    "====================", CR
                        byte    CR
                        byte    "Joy X...            ", CR 
                        byte    "Joy Y...            ", CR 
                        byte    "Acc X...            ", CR 
                        byte    "Acc Y...            ", CR 
                        byte    "Acc Z...            ", CR 
                        byte    "Btn C...            ", CR 
                        byte    "Btn Z...            ", CR
                        byte    0

CC_Screen               byte    "===============================", CR
                        byte    " Wii Classic Controller Reader ", CR
                        byte    "===============================", CR
                        byte    CR
                        byte    "L-Joy X...       R-Joy X...    ", CR 
                        byte    "L-Joy Y...       R-Joy Y...    ", CR
                        byte    "L-Trig....       R-Trig....    ", CR
                        byte    "L-Btn Z...       R-Btn Z...    ", CR
                        byte    CR
                        byte    "      |                 x      ", CR
                        byte    "    -- --    - H +    y   a    ", CR
                        byte    "      |                 b      ", CR
                        byte    0


pub main | check, t

  setup_io                                                      ' configure HC-8 io pins
  term.start(RX1, TX1, %0000, 115_200)                          ' start terminal
  pause(1)

  term.tx(CLS)

  check := ina[SD_CLK..SD_DO]
  if (check <> %11)                                             ' option switches off? 
    term.str(string("Warning: Option switche(s) ON")) 
    repeat                      
      waitcnt(0)                                                ' stop here
  

  ' try nunchuk first

  nc.start(WII_CLK, WII_DAT)                                    ' start nc controller     
  if (nc.device == nc#NUNCHUK)
    nc_demo

  ' if no nunchuk, try classic

  cc.start(WII_CLK, WII_DAT)                                    ' start cc controller     
  if (cc.device == cc#CLASSIC)
    cc_demo

  ' whoops...
  
  term.str(string("Unknown or no controller attached."))      ' display error msg
  repeat                                                      ' force reset
    waitcnt(0)


pub nc_demo | t

'' Scan and display values from Nunchuk controller

  term.tx(CLS)
  term.str(@NC_Screen)     

  t := cnt
  repeat
    nc.scan                                                     ' get fresh values
    show_value(nc.joy_x,  9,  4, 4)                             ' x joystick
    show_value(nc.joy_y,  9,  5, 4)                             ' y joystick
    show_value(nc.acc_x,  9,  6, 4)                             ' x accelerometer
    show_value(nc.acc_y,  9,  7, 4)                             ' y accelerometer
    show_value(nc.acc_z,  9,  8, 4)                             ' z accelerometer       
    show_value(nc.btn_c,  9,  9, 4)                             ' c button
    show_value(nc.btn_z,  9, 10, 4)                             ' z button
    
    waitcnt(t += (50 * MS_001))                                 ' refresh every 50ms
 

pub cc_demo | t

'' Scan and display values from Classic controller

  term.tx(CLS)
  term.str(@CC_Screen)

  t := cnt
  repeat
    cc.scan                                                     ' get fresh values
    show_value(cc.ljoyx, 12,  4, 2)                             ' lx joystick
    show_value(cc.ljoyy, 12,  5, 2)                             ' ly joystick  
    show_value(cc.rjoyx, 29,  4, 2)                             ' rx joystick
    show_value(cc.rjoyy, 29,  5, 2)                             ' ry joystick 
    show_value(cc.ltrig, 12,  6, 2)                             ' l trigger
    show_value(cc.rtrig, 29,  6, 2)                             ' r trigger
    show_value(cc.lbtnz, 12,  7, 2)                             ' l z button
    show_value(cc.rbtnz, 29,  7, 2)                             ' r z button
                                                                 
    ' graphic display of top buttons                             
                                                                 
    show_state(cc.dpad_up,   "|", "*",  6,  9)                  ' direction pad up
    show_state(cc.dpad_dn,   "|", "*",  6, 11)                  ' direction pad down
    show_state(cc.dpad_lf,   "-", "*",  4, 10)                  ' direction pad left
    show_state(cc.dpad_rt,   "-", "*",  8, 10)                  ' direction pad right

    show_state(cc.btn_minus, "-", "*", 13, 10)
    show_state(cc.btn_home,  "H", "*", 15, 10)
    show_state(cc.btn_plus,  "+", "*", 17, 10)

    show_state(cc.btn_x,     "x", "*", 24,  9)    
    show_state(cc.btn_y,     "y", "*", 22, 10)
    show_state(cc.btn_a,     "a", "*", 26, 10)
    show_state(cc.btn_b,     "b", "*", 24, 11)

    show_state(cc.btn_l,     ".", "*",  6,  6)
    show_state(cc.btn_r,     ".", "*", 23,  6)    

    waitcnt(t += (50 * MS_001))                                 ' refresh every 50ms

  
pub show_value(value, x, y, width)

'' Display right-justied value at X/Y

  move_to(x, y)
  term.rjdec(value, width, " ")


pub show_state(state, ch0, ch1, x, y)

'' Show state (false or true) using characters
'' -- ch0 displayed when state is 0
'' -- ch1 displayed wehn state is non-zero 

  move_to(x, y)
  if (state == 0)
    term.tx(ch0)
  else
    term.tx(ch1)  
  

pub move_to(x, y)

'' Move PST cursor to X/Y 

  term.tx(GOTOXY)
  term.tx(x)
  term.tx(y) 
  

con

  { ================================= }
  {                                   }
  {  S U P P O R T   R O U T I N E S  } 
  {                                   }
  { ================================= }


var

  long  lastscan                                                ' ttl/DMX input scan


pub setup_io

'' Configure for basic IO control

  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode                                  
  low(OUT_EN)                                                   ' enable 74x245 

  high(LD_165)                                                  ' setup x165 io pins
  low(CLK_165)
  input(DO_165)
  
  set_rs485(NOCON)                                              ' RS-485 off


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be at least 1
    return
  else
    t := cnt - 1792                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)


pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles output pin

  !outa[pin]                                                    ' invert state
  dira[pin] := 1  
  

pub input(pin)

  dira[pin] := 0

  return ina[pin]


pub ttl_pin | btns

'' Reads TTL header for 1-of-8 input
'' -- returns -1 (no input)
'' -- returns 0 to 7 for input on IN0 to IN7
''    * only one input allowed

  btns := ttl_inputs(true)                                      ' read ttl inputs

  if (bit_count(btns) == 1)                                     ' only one button pressed?
    return bit_pos(btns, 0)                                     '  yes, return button #
  else
    return -1                                                   '  none or bad input


pub ttl_inputs(rescan) 

'' Return TTL inputs status
'' -- updates global var "lastscan" if rescan is true

  if (rescan)
    lastscan := x165_in(16, MSBFIRST) | (ina[DMX_A8] << 16)     ' read in0..in7, dmx addresss
   
  return (lastscan & $FF)                                       ' return TTL input bits


pub dmx_address(rescan) 

'' Return DMX address switch setting
'' -- updates global var "lastscan" if rescan is true

  if (rescan)
    lastscan := x165_in(16, MSBFIRST) | (ina[DMX_A8] << 16)     ' read in0..in7, dmx addresss

  return (lastscan >> 8) & $01FF                                ' return DMX address bits
 

pub x165_in(bits, mode) | tmp165

'' Input value from 74x165(s)      
                                                                                          
  outa[LD_165] := 0                                             ' blip Shift/Load line    
  outa[LD_165] := 1                                                                         
                                                                                          
  tmp165 := 0                                                   ' clear workspace
  bits := 1 #> bits <# 32                                       ' limit to legal value         
  repeat bits                                                   ' read n bits         
    tmp165 := (tmp165 << 1) | ina[DO_165]                       ' get new bit             
    outa[CLK_165] := 1                                          ' blip clock              
    outa[CLK_165] := 0  

  if (mode == LSBFIRST)                                         ' LSBFIRST result?
    tmp165 ><= bits                                             ' reverse bits
   
  return tmp165
  

pub option_inputs

'' Returns state of option switch bits (%0000..%1111, all off to all on)

  dira[OPT_BR..OPT_SM] := %0000                                 ' force to input mode

  return !ina[OPT_BR..OPT_SM]                                   ' return state (1 = on)


pub set_rgled(state)

'' Sets R/G LED
'' -- direct control only
'' -- for red/green/yellow use "efx_rgled" driver and methods

  case state
    OFF : outa[R_LED..G_LED] := %00
    RED : outa[R_LED..G_LED] := %10
    GRN : outa[R_LED..G_LED] := %01
    YEL : outa[R_LED..G_LED] := %00                             ' yellow requires PASM driver

  dira[R_LED..G_LED] := %11
      

pub set_rs485(state)

'' Sets RS-485 RX and TX enable inputs

  case state
    NOCON : outa[RX2_EN..TX2_EN] := %10
    RX    : outa[RX2_EN..TX2_EN] := %00 
    TX    : outa[RX2_EN..TX2_EN] := %11   
    RXTX  : outa[RX2_EN..TX2_EN] := %01

  dira[RX2_EN..TX2_EN] := %11


pub bit_count(value) | bc

'' Returns # of bits set (1) in value

  bc := 0                                                       ' clear count
  repeat 32                                                     ' test all bits
    bc += (value & %1)                                          ' add bit value to count
    value >>= 1                                                 ' next bit

  return bc


pub bit_pos(value, mode)

'' Returns position of 1st bit set (0..31)
'' -- mode 0 to scan from lsb, mode 1 to scan from msb 
'' -- -1 = no bits set

  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1

  else
    if (mode == 0)                                              ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)
    else
      return (>|value - 1)


pub bit_val(value, pos)

'' Returns bit (0..1) from pos (0..31) in value 

  if ((pos => 0) and (pos =< 31))
    return (value >> pos) & 1
  else
    return 0

    
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      